/*-------------------------------------------------------------------------
 *
 * This is part of: libpq-fe.h
 *	  This file contains definitions for structures and
 *	  externs for functions used by frontend postgres applications.
 *
 * Copyright (c) 1994, Regents of the University of California
 *
  Permission to use, copy, modify, and distribute this software and
  its documentation for any purpose, without fee, and without a written
  agreement is hereby granted, provided that the above copyright notice
  and this paragraph and the following two paragraphs appear in all
  copies.

  In no event shall the University of California be liable to
  any party for direct, indirect, special, incidental, or consequential
  damages, including lost profits, arising out of the use of this
  software and its documentation, even if the University of California
  has been advised of the possibility of such damage.

  The University of California specifically disclaims any
  warranties, including, but not limited to, the implied warranties
  of merchantability and fitness for a particular purpose.
  The software provided hereunder is on an "as-is" basis, and
  the University of California has no obligations to provide
  maintainance, support, updates, enhancements, or modifications.

 * $Id: libpq-fe.h,v 1.50 1999/05/25 16:15:13 momjian Exp $
 *
 *-------------------------------------------------------------------------
 */

#define ERROR_MSG_LENGTH 4096
#define CMDSTATUS_LEN 40

/* typedef struct _IO_FILE FILE;  */

typedef unsigned int Oid;

typedef union SockAddr
{
  /*         struct sockaddr sa;  */
  /*         struct sockaddr_in in;  */
  /*         struct sockaddr_un un;  */
        struct sockaddr int;
        struct sockaddr_in int;
        struct sockaddr_un int;
} SockAddr;

typedef enum
{
	CONNECTION_OK,
	CONNECTION_BAD
} ConnStatusType;

typedef struct Dlelem
{
        struct Dlelem *dle_next;        /* next element */
        struct Dlelem *dle_prev;        /* previous element */
        void       *dle_val;            /* value of the element */
        struct Dllist *dle_list;        /* what list this element is in */
} Dlelem;

typedef struct Dllist
{
        Dlelem     *dll_head;
        Dlelem     *dll_tail;
} Dllist;

typedef struct pgLobjfuncs
{
  Oid   fn_lo_open; /* OID of backend function lo_open		*/
  Oid	fn_lo_close;/* OID of backend function lo_close		*/
  Oid	fn_lo_creat;/* OID of backend function lo_creat		*/
  Oid	fn_lo_unlink;		/* OID of backend function * lo_unlink	*/
  Oid	fn_lo_lseek;/* OID of backend function lo_lseek		*/
  Oid	fn_lo_tell; /* OID of backend function lo_tell		*/
  Oid	fn_lo_read; /* OID of backend function LOread		*/
  Oid	fn_lo_write;/* OID of backend function LOwrite		*/
} PGlobjfuncs;

typedef enum
{
	PGRES_EMPTY_QUERY = 0,
	PGRES_COMMAND_OK,	/* a query command that doesn't return */
		      /* anything was executed properly by the backend */
	PGRES_TUPLES_OK,	/* a query command that returns tuples */
		     /* was executed properly by the backend, PGresult */
		     /* contains the resulttuples */
        PGRES_COPY_OUT,
	PGRES_COPY_IN,
	PGRES_BAD_RESPONSE,	/* an unexpected response was recv'd from
							 * the backend */
	PGRES_NONFATAL_ERROR,
	PGRES_FATAL_ERROR

} ExecStatusType;

/* PGconn encapsulates a connection to the backend */
typedef struct pg_conn
{
   char	   *pghost;	/*the machine on which the server is running */
   char	   *pgtty;   /*tty on which the backend message is displayed */
   char	   *pgport;	/* the communication port with the backend */
   char	   *pgoptions;	/* options to start the backend with */
   char	   *dbName;		/* database name */
   ConnStatusType status;
   char		errorMessage[ERROR_MSG_LENGTH];
   /* pipes for be/fe communication */
   FILE	   *Pfin;
   FILE	   *Pfout;
   FILE	   *Pfdebug;
   int			sock;		/* The socket */
   SockAddr	laddr;		/* Local address */
   SockAddr	raddr;		/* Remote address */
   char		salt[2];
   int			asyncNotifyWaiting;
   Dllist	   *notifyList;
   char	   *pguser;		/* Postgres username of user who is
       						 * connected */
   char	   *pgpass;
   PGlobjfuncs *lobjfuncs; /* Backend function OID's for large object
   						 * access */
} PGconn;

typedef struct pgresAttDesc
{
  char	   *name;		/* type name */
  Oid	   adtid;		/* type id */
  short	   adtsize;	/* type size */
} PGresAttDesc;

typedef struct pgresAttValue
{
  int len;		/* length in bytes of the value */
  char *value;		/* actual value */
} PGresAttValue;

typedef struct pg_result
{
  int ntups;
  int numAttributes;
  PGresAttDesc *attDescs;
  PGresAttValue **tuples; /* each PGresTuple is an array of
			   * PGresAttValue's */
  int tupArrSize; /* size of tuples array allocated */
  ExecStatusType resultStatus;
  char	cmdStatus[CMDSTATUS_LEN];	/* cmd status from the
					 * last insert query */
  int binary;		/* binary tuple values if binary == 1,
			 * otherwise ASCII */
  PGconn *conn;
} PGresult;

	typedef char pqbool;

	typedef struct _PQprintOpt
	{
		pqbool		header;		/* print output field headings and row
								 * count */
		pqbool		align;		/* fill align the fields */
		pqbool		standard;	/* old brain dead format */
		pqbool		html3;		/* output html tables */
		pqbool		expanded;	/* expand tables */
		pqbool		pager;		/* use pager for output if needed */
		char	   *fieldSep;	/* field separator */
		char	   *tableOpt;	/* insert to HTML <table ...> */
		char	   *caption;	/* HTML <caption> */
		char	  **fieldName;	/* null terminated array of repalcement
								 * field names */
	} PQprintOpt;

/*extern int PQprint(FILE *fout,		/* output stream */
/*			PGresult *res,
/*			PQprintOpt *ps);	/* option structure */

/* Accessor functions for PGconn objects */
	extern char *PQdb(PGconn *conn);
	extern char *PQuser(PGconn *conn);
	extern char *PQpass(PGconn *conn);
	extern char *PQhost(PGconn *conn);
	extern char *PQport(PGconn *conn);
	extern char *PQtty(PGconn *conn);
	extern char *PQoptions(PGconn *conn);
	extern ConnStatusType PQstatus(PGconn *conn);
	extern char *PQerrorMessage(PGconn *conn);
	extern int	PQsocket(PGconn *conn);
	extern int	PQbackendPID(PGconn *conn);
/* Accessor functions for PGresult objects */
	extern ExecStatusType PQresultStatus(PGresult *res);
	extern const char *PQresStatus(ExecStatusType status);
	extern char *PQresultErrorMessage(PGresult *res);
/* 	extern const char *PQresultErrorMessage(PGresult *res);  */
	extern int	PQntuples(PGresult *res);
	extern int	PQnfields(PGresult *res);
	extern int	PQbinaryTuples(PGresult *res);
	extern char *PQfname(PGresult *res, int field_num);
	extern int	PQfnumber(PGresult *res, const char *field_name);
 	extern Oid      PQftype(PGresult *res, int field_num);  
/*  	extern int      PQftype(PGresult *res, int field_num);    */
	extern int	PQfsize(PGresult *res, int field_num);
	extern int	PQfmod(PGresult *res, int field_num);
	extern char *PQcmdStatus(PGresult *res);
	extern const char *PQoidStatus(PGresult *res);
	extern const char *PQcmdTuples(PGresult *res);
	extern char *PQgetvalue(PGresult *res, int tup_num, int field_num);
	extern int	PQgetlength(PGresult *res, int tup_num, int field_num);
	extern int	PQgetisnull(PGresult *res, int tup_num, int field_num);

/* Delete a PGresult */
	extern int PQclear(PGresult *res);

